/*
   This scene is (c) 1995 Han-Kwang Nienhuys

   Distributed under terms of the GPL.
*/

/*

  composite { conebend}

  |\____
  | ____======= \
  |/           ||
	     == /

  Reference point: center line of valve #1, at height of connection between tube and valve

*/


/*
  Data of Vincent Bach Stradivarius Model 43 trumpet:

  (x=0 at widest point)

  New Empirical formula (advanced):

  r   = ax^3 + bx^2 + cx + d
  r^2 = y^2 + z^2

  values for a1..3, b1..3, c1..3, d1..3 below

   Different values for

     0<=x<=59; (a1-d1)
     59<=x<=170.5 (a2-d2)
     170.5<=x<=439.8 (a3-d3)

  Cone around x-axis, ends at x=0 (widest), x=439.8 (smallest, r=7.2mm)



 ( Old Empirical formule (simple):

    r   =  1/(a(x-12)+b) + r0
    r^2 =  4*(y^2+z^2)

    a = 0.000327
    b = 0.0119
    r0= 7.8120

 )



*/

#define a1 = -0.0001282643
#define b1 =  0.0247716406
#define c1 = -1.68585659
#define d1 = 58.7765925

#define a2 = -273.029E-9
#define b2 =  0.0003958894
#define c2 = -0.1461851944
#define d2 = 26.5016728

#define a3 =-14.7486E-9
#define b3 =  0.0000781595
#define c3 = -0.0601439641
#define d3 = 26.5016728

#declare coneshapea = object {
    algebraic {
        $ y^2+z^2 = (a1*x^3 + b1*x^2 + c1*x + d1)^2 $
        sturm
    }
    bounded_by { box { <0, 65, 65>, <59, -65, -65> } }
    clipped_by { bounded_by }
}

#declare coneshapeb = object {
    algebraic {
        $ y^2+z^2 = (a2*x^3 + b2*x^2 + c2*x + d2)^2 $
        sturm
    }
    bounded_by { box { <59, 20, 20>, <170.5, -20, -20> } }
    clipped_by { bounded_by }
}

#declare coneshapec = object {
    algebraic {
        $ y^2+z^2 = (a3*x^3 + b3*x^2 + c3*x + d3)^2 $
        sturm
    }
    bounded_by { box { <170.5, 12, 12>, <439.8, -12, -12> } }
    clipped_by { bounded_by }
}

#define r0 = 7.8120
#define a = 0.000327
#define b = 0.0119

/* conesimp: simple cone, only one 4th degree surface */

#declare conesimp = composite {
    object {
        algebraic {
            $ (1 + r0 * (a * y + b))^2 = 4 * (x^2 + z^2) * (a * y + b)^2 $
            sturm
        }
        translate y*12
        rotate -90*z
        bounded_by {
            union {
                box { <0, 70, 70>, <100, -70, -70> }
                box { <100, 17, 17>, <439.8, -17, -17> }
            }
        }
        clipped_by { bounded_by }
    }
    torus {
        67.6818, 3 // r(x=0) + 3*cos (arctan (-(dr/dx)/2))
        rotate z*90
        translate x*2.796 // bla bla 3*sin bla bla
    }
}

// coneadv: advanced cone, composed of three 6th degree surfaces

#declare coneadv = composite {
    object { coneshapea }
    object { coneshapeb }
    object { coneshapec }
    torus { 60.307, 3.0 rotate z*90 translate 2.580*x }
}

#declare ca=-2.32863E-6
#declare cb=0.0002951544
#declare cc=-0.0000294921
#declare cd=6.5

#declare connbendvalve = object {
// buis, r=6.5 op x=0, r=7.2 op x=83.8    
    intersection {
        object { QCone_X scale <778.14, 6.5, 6.5> }
        plane {-x, -778.14 }
        plane {x, 861.94 }
        translate  -778.14*x
    }
    bounded_by{box{<0, 8, 8>,< 84, -8, -8> } }
}


#declare conebend = composite {
    
    object { conesimp }  // ref temp. at end of cone
    
    composite {  // the bend itself...
        
        intersection {
            torus { 45, 7.2 rotate x*90 }
            plane { y, 0 rotate z*26.339 }  plane { -y, 0 rotate -z*26.339 }
            translate -13*x
        }
        
        intersection {
            torus {30.494, 7.2 rotate 90*x }
            plane {-y, 0 rotate z*26.339 } plane {-x, 0 }
            translate 6.436*y
        }
        
        intersection {
            torus {30.494, 7.2 rotate 90*x }
            plane {y, 0 rotate -z*26.339 } plane {-x, 0 }
            translate -6.436*y
        }
        translate <439.8, -36.93, 0>
    }
    
    translate <-439.8+102.8, 73.87, 0>  // Reference point is end of tube, center of valve /hmm
    rotate -x*12.87
    
    object {connbendvalve translate 19*x }
    sphere {<15, 0, 0>, 9 }
    
    intersection {
        torus { 6.5, 6.5 }
        plane { x, 0 }
        plane { <-0.7071067, 0, 0.7071067>, 0 }
        translate <10, 0, 6.5>
    }
    
    translate -z*6.5
}


