/*
   ibuf.cc -- intersection buffers.

   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
 */

#include	"ray.hh"
#include	"object.hh"
#include	"proto.hh"
#include	"ibuf.hh"
#include 	"extern.hh"


// to reduce the remax() calls, try to make them queues big enough
static int biggest_queue = 20;

/* add intersection queue */
void
ibuf::add_to(int flags, dqueue const & q)
{
    if (flags & CHKALL)
    {
//	sorted = false;
        if (len >= max)
	    remax(max * 2 + 1);

	// add entry.
        list[len++] = q;
    }

    /* exponential falloff of light in transparent media */
    if (flags & NO_TRANSUPDATE)
    {
        if (q.obj->flags & O_TRANSPARENT)
            return ;
    }
    if (q.t < minq.t)
        minq = q;
}

void ibuf::add_to(int fl, flt t, object *io, bool ent, object *io2)
{
    add_to(fl,dqueue(t, io, ent, io2));
}

void ibuf::remax(int nmax)
{
    if (nmax > biggest_queue)
	biggest_queue = nmax;

    dqueue *olist = list;
    list = new dqueue[nmax];

    len = (len < nmax) ? len : nmax;
    for (int i = 0; i < len; i++)
	list[i] = olist[i];
    
    max = nmax;
    delete[]  olist;
}

ibuf::ibuf() {
    list = new dqueue[biggest_queue];
    max = biggest_queue;

    init();
}


ibuf::~ibuf() {
    delete[]  list;
}


/* needed for sorting */
PRIVATE int
        q_compare(const void *q1, const void *q2)
{
    return     SGN(((dqueue const *) q1)->t - ((dqueue const *) q2)->t );
}

/* sort intersection buffer */
void
ibuf::sort()
{
    if (!sorted)
    {
        qsort(list, len, sizeof(dqueue), q_compare);
        sorted = true;
    }
}

/* print intersection buffer */
void
ibuf::print() const
{
#ifdef DEBUG
    printf("ibuf: sort %c len %d ", (sorted)?'T':'F',len);
    if (len)
    {
        for (int     i = 0; i < len; i++){
	    if (list[i].entering) dprintf("enter ");
            dprintf("%f, ", list[i].t);
	}

        dprintf("\n");
    } else if (minq.obj)
    {
        dprintf("%f\n", minq.t);
    }
#endif          /* DEBUG */
}
