/*
   intersect.cc -- main intersection routines, ray::*

   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
 */

#include	<math.h>
#include	"ray.hh"
#include	"proto.hh"
#include 	"extern.hh"
#include	"ibuf.hh"
#include 	"object.hh"
#include 	"light.hh"
#include 	"grid.hh"
#include 	"stats.hh"

/*
 * intersect(): intersect everything in the whole scene, and then fill in
 * the geometric intersection details: normal, speed effects
 */
 intersection
Scene::all_intersections(ray & r)
{
    /* intersect everything in scene */
    ibuf   q;
    bool    isinside;
    the_object->all_intersections(&q, &r, 0, isinside);

    intersection i;
    i.r = &r;
    i.q_ent = q.min();


    object *op = i.q_ent.obj;
    if (!op)
    {
        i.q_ent.t = INFTY;
        return i;               /* nothing was found */
    }

    r.line.rayintersections++;
    vector  moved = r.now * op->getspeed();

    /* determine the intersection point.
     *
     * i.x = t*transray.dir + transray.pos + obj->speed * Time = t*r.dir + r.pos
     *
     */

    i.objectx = r.point(i.q_ent.t);
    i.x = i.objectx + moved;

    /* determine normal at intersection point */
    i.n = i.q_ent.obj->normal(i.objectx);

    for (object *dp = i.q_ent.obj->daddy; dp; dp = dp->daddy)
	dp->transformn(i.n);

    i.objectn = i.n;
    return i;
}

/****************************************************************/

void
ray::operator=(ray const & src)
{
    rayid = lastrayid++;

    sender =src.sender;
    pos =src.pos;
    dir =src.dir;
    type=src.type;  	
    maxt=src.maxt;
    mint=src.mint;       
}

void
ray::print() const
{
#ifdef DEBUG
    printf("type %d ID %ld (last %ld) t range [%f,%f]\n", type, rayid, lastrayid, mint, maxt);
    dir.print("dir");
    pos.print("pos");
#endif
}


ray::ray(raytype newraytype, flt time, object *o)
{
    now = time;
    maxt = INFTY;
    mint = 0;
    type = newraytype;
    rayid = lastrayid++;
    sender = o;
    line.rays++;

    switch (newraytype)
    {
    case R_EYE:
        line.eye++;
        break;
    case R_REFLECT:
        line.reflected++;
        break;
    case R_REFRACT:
        line.refracted++;
        break;
    case R_SHADOW:
        line.shadow++;
        break;
    default:
        assert(false);
    }
}


/*
  DESCRIPTION - print stats of one scanline, reset stats
*/
void
ray::reset_stats()
{
    printf("rays shot: %ld, hits: %ld, rel: %.2f, ray/pxl: %.2f\n"
           "          eye: %ld shad: %ld, refl: %ld, refr %ld, depth %d/%d\n",
           line.rays, line.rayintersections, (flt) line.rayintersections / line.rays,
           (flt) line.rays / xres, line.eye, line.shadow, line.reflected,
           line.refracted, line.maxdepth, the_scene.max_trace_level);

    stats.shadow += line.shadow;
    stats.reflected += line.reflected;
    stats.refracted += line.refracted;
    stats.eye += line.eye;

    stats.rays->test += line.rays;
    stats.rays->hit += line.rayintersections;


    line.shadow = line.reflected =
        line.refracted = line.eye =
        line.rays = line.rayintersections = line.maxdepth = 0;
}


raystatstruct
ray::stats =
{
    0, 0, 0, 0, shape_stats::regist("ray", -1)
};


linestatstruct
  ray::line;
long ray::lastrayid;
