/*
   message.c -- input/output of messages, debugging dumps

   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
 */

#include <stdarg.h>
#include <string.h>

#include	"ray.hh"
#include	"proto.hh"
#include 	"extern.hh"
#include	"vray.hh"

#define 	INDENTAMOUNT 2
#define MAXSTRING 100

PUBLIC char *
        xstrdup(const char *s)
{
    char   *ptr = new char[strlen(s) + 1];
    strcpy(ptr, s);
    return ptr;
}

/* prints an error message with linenumber to error_out and then dies */
void
errormsg(const char *format,...)
{
    char    s1[MAXSTRING];    
    va_list arglist;
    sprintf(s1, "\nerror: ");

    va_start(arglist, format);
    char s2[MAXSTRING];
    vsprintf(s2, format, arglist);
    va_end(arglist);

    fprintf(error_out, "%s%s\n", s1, s2);
    exit(1);
}

/* prints a warning with linenumber to stdout */
void
warning(const char *format,...)
{
    char s1[MAXSTRING];
    va_list arglist;
    sprintf(s1, "\nwarning: ");
    char s2[MAXSTRING];
    va_start(arglist, format);
    vsprintf(s2, format, arglist);
    va_end(arglist);

    fprintf(error_out, "%s%s\n", s1, s2);
}


inline bool isopen(char c)
{
    switch(c) {
        case '<':
        case '(':
        case '{':
        case '[':
	return true;
	default: return false;
    }
}

inline bool isclose(char c)
{
    switch(c) {
        case ']':
        case '}':
        case '>':
        case ')':
	return true;
	default: return false;
    }
}



/* a printf with indent before each line according to {}()[] */
void
dprintf(const char *format,...)
{
    va_list arglist;


    va_start(arglist, format);
    char    s[1000]; // ugh.
    vsprintf(s, format, arglist);
    va_end(arglist);

    static int indent = 0;
    static bool gotnl=true;
    char line[2000], *lp = line;
    for (char *cp = s; *cp; cp++) {
	if (isclose(*cp))
	    indent -= INDENTAMOUNT;
	if (gotnl) {
	    for (int i=0; i<indent; i++)
		*lp++ = ' ';
	}
	if (*cp == '\n')
	    gotnl = true;
        else
	    gotnl =false;


	*lp++ = *cp;
	if (isopen(*cp))
	    indent += INDENTAMOUNT;
    }
    *lp = 0;
    fputs(line,error_out);
}

/****************************************************************
  OPENING/CLOSING FILES
 ****************************************************************/

static svec<char *> path;

void
add_path(const char *s)
{
    path.add(xstrdup(s));
}

/* open file, searching library paths */
FILE *open_file(const char *name, const char *mode, bool abrt = true)
{
    char    s[1000]; 	//lelijk.

    strcpy(s, name);
    FILE   *    f = fopen(s, mode);
    for (    int     i = 0; !f && i < path.sz(); i++)
    {
        strcpy(s, path[i]);
        strcat(s, PATHSEP);
        strcat(s, name);
        f = fopen(s, mode);
    }

    if (!f && abrt)
        errormsg("Cannot find file %s", name);

    return f;
}

void
close_file(FILE * f)
{
    fclose(f);
}
