/*
   pix.cc -- pixel data for imagemaps
   
   (c) 1993, 1994, 1995 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>

   this is part of Rayce, a raytracer based on "Ray" by George Kyriazis	
   <kyriazis@turing.cs.rpi.edu> Rayce is released under the terms of the
   GNU General Public License, it has no warranty what so ever. For more
   details: see the file "COPYING"
*/

#include "proto.hh"
#include "texture.hh"
#include "vray.hh"
#include "pixel.hh"


/* save info about previously read maps */
svec<pixel_data *> themaps;
void pixel_data::print() const
{
        #ifdef DEBUG
    dprintf("%s: %d %d (refs %d)\n",name, imagex,imagey, refs);
            #endif
}

pixel_data::pixel_data(const char *nm)
{    
    refs = 1;
    imagex=imagey = 0;
    name = xstrdup(nm);
    printf("[%s",nm);
}

pixel_data::~pixel_data()
{
    delete[] name;
    assert(refs ==1); 
}
color
pixel_data::get_color(flt u, flt v) const 
{
    int x_pix = int (u * imagex);
    int y_pix = int (v * imagey);

    return get_byte_color(x_pix, y_pix);
}

/*
 * Bilinear interpolation (thanks to Advanced Animation and Rendering
 * Techniques by Watt & Watt)
 */
color
pixel_data::interpolate(flt u, flt v) const
{
    assert(u >= 0.0 && v >= 0.0 && v <= 1.0 && u <= 1.0);

    /* relative position to pixels */
    flt pixu = u * (flt) imagex;
    flt pixv = v * (flt) imagey;

    int right_coord = int (ceil(pixu));
    if (right_coord >= imagex)
        right_coord = 0;

    int up_coord = int (ceil(pixv));
    if (up_coord >= imagey)
        up_coord = 0;

    /* find colors in the corners */
    color leftup=    get_byte_color(int (floor(pixu)), up_coord); ;
    color leftdown=    get_byte_color( int (floor(pixu)), int (floor(pixv)));
    color rightup=    get_byte_color( right_coord, up_coord);
    color rightdown=    get_byte_color( right_coord, int (floor(pixv)));


    /* don't walk off the picture */
    pixu = pixu - ceil(pixu);
    if (pixu < 0.0)
        pixu += 1.0;

    pixv = pixv - ceil(pixv);
    if (pixv < 0.0)
        pixv += 1.0;

    /* find colors above and below of (u,v) */
    color up = ((1.0 - pixu) * leftup) + (pixu * rightup);
    color down = ((1.0 - pixu) * leftdown) + (pixu * rightdown);

    /* and linearly interpolate them */
    return (1.0 - pixv) * down + pixv * up;
}

/****************/
colmapped_data::colmapped_data(const char *s) : pixel_data(s) 
{
     pal = 0;
     data=0;
}


byte_color
colmapped_data::get_byte_color(int x, int y) const 
{
    int	offset = (x + y * imagex) % (imagex * imagey);       
    int	palent = data[ offset];
    return pal[palent];
}

/****************/

truecol_data::truecol_data(const char *s) : pixel_data(s) 
{
     data = 0;
}

//  get a 24 bit color at (x,y) 
byte_color
truecol_data::get_byte_color(int x, int y) const 
{
    int	offset = (x + y * imagex) % (imagex * imagey);
    
    /* no palette, rgb raw data */
    return data[ offset];
}

// search table of previously loaded pixeldatas
pixel_data *
search_pixels(const char *fn)
{
    for (int i = 0; i < themaps.sz(); i++)
    {
        if (!strcmp(themaps[i]->name, fn))
        {
            themaps[i]->refs++;
            return themaps[i];
        }
    }
    return 0;
}
void
add_pixels(pixel_data *p)
{
    themaps.add(p);
}
/****************************************************************
  TARGA
 ****************************************************************/
void
read_TGA_head(FILE * f, int &x, int &y)
{
    /* read TGA header */
    int     header[18];
    for (int i = 0; i < 18; i++)
        header[i] = getc(f);

    /* check file id */
    if (header[2] != 0x02 || header[16] != 24 || header[17] != 32)
        errormsg("Not a valid TGA!");

    /* calc resolution */
    x = header[12] + header[13] * 256;
    y = header[14] + header[15] * 256;
}


pixel_data *
        get_TGA_pixels(char *fn)
{    
    pixel_data *q = search_pixels(fn);
    if (q) 
	return q;
   
    truecol_data *p = new truecol_data(fn);
    add_pixels(p);

    /* open file, buffer it */
    FILE   *f = open_file(fn, "rb");
    char   *buf = new char[BUFFERSIZE];
    if (setvbuf(f, buf, _IOFBF, BUFFERSIZE))
        errormsg("cannot buffer TGA input");    
    
    read_TGA_head(f, p->imagex, p->imagey);
    long  imsiz = p->imagex * p->imagey;
    
    byte_color *cp = p->data = new byte_color[imsiz];
    


    for (long l = 0; l < imsiz; l++)
    {
	int b = getc(f);
	int g = getc(f);
	int r = getc(f);
    
	*cp++ = byte_color((byte) r,(byte) g,(byte) b);
    }

    close_file(f);
    delete[]  buf;
    printf("]");
    return p;
}


/* eof */

