/*
   plane.cc -- infinite plane thingies.
 

   (c) 1993, 1996 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>
 */



#include "object.hh"
#include "shapes.hh"
#include "proto.hh"
#include "extern.hh"


#include "ibuf.hh"

int
plane_data::do_intersection(ray * r, flt * t, int ) const
{
    /* solve plane/ray equation: ((t*dir + pos) * (normal)) = mov */
    flt         d = n * r->dir;

    if (ISZERO(d))              /* plane is parallel with ray */
        return 0;

   *t = (mov - n * r->pos) / d;

    return (*t > tolerance );
}

vector  plane_data::normal(vector) const
{
    return n;
}

void
plane_data::translate(vector v)
{
    mov += v * n;
}

void
plane_data::rotate(const matrix & rotmat)
{
    n.rotate(rotmat);
}
bool
plane_data::inside(vector x) const
{
    return (x * n < mov);
}
/*
 * afgespiekt van pov source
 */
void
plane_data::scale(const vector s)
{
    n.x /= s.x;
    n.y /= s.y;
    n.z /= s.z;
    flt     len = n.length();
    mov /= len;
    n /= len;
}

shape_stats *plane_data::stat = shape_stats::regist("plane");
plane_data::plane_data(vector in, flt m)
{

    type = PLANE;
    n=in;
    mov = m;
}

void
plane_data::print() const
{
#ifdef DEBUG
    n.print("normal");
    dprintf("translation %f\n", mov);
#endif          /* DEBUG */
}


void
plane_data::precompute()
{
    stat->howmuch++;
    prims++;
}


PUBLIC object *
        get_new_plane_object(vector inn, flt r)
{
    object *o = new object;
    o->data = new plane_data(inn,r);
    o->data->setdaddy(o);
    o->flags = O_INFINITE | O_CLOSED ;

    return o;
}
