/*
 * proto.h -- function prototypes
 */
#ifndef PROTO_HH
#define PROTO_HH

#include "ray.hh"
enum {
    DEBUGTOK = 1,           /* print tokens as they are read */
    DEBUGPARS = 2,          /* turn yydebug on */
    DEBUGRUNTIME = 4,      /* make solve.c shut up. */
    DEBUGINTERIOR = 8,      /* print everything in scene before tracing and after parsing */
    DEBUGDECL = 16,         /* print everything which is declared */
    DEBUGSELFCHECK = 32    /* miscellaneous self checks. */
};


#ifndef PATHSEP
#define PATHSEP "/"
#endif


/****************************************************************
   TYPENAMES
 ****************************************************************/
struct bounding_box;
struct image_map;
struct shape_stats;
struct lightbuf;
struct intersection;
struct object;
class matrix;
struct vector;
struct Color;
typedef Color color;
struct nc_color;
struct Identifier;
struct ray;
struct ibuf;
struct texture;
struct Optilist;
struct dqueue;
struct Camera;
struct Grid;
struct Grid_tree;
struct pixel_data;
struct poly_instruction;
struct object_list_optimisation;
struct object_list;
struct simple_texture;
struct local_finish;
struct pigment_modifier;
struct composite_data;
template<class T>
class svec;

#include "vector.hh"

/****************************************************************/


/* miscellaneous routines */
void raytrace(const char *outname);
void readfile(const char *fname);
void abort_trace(int);

FILE *open_file(const char *name, const char *mode, bool abort = true);
void close_file(FILE * f);
void add_path(const char *s);

/* device dep. code */
void check_abort();
void init_machine();
void initialize();
void timer_start();
long timer_stop();

/* errors */
void warning(const char *format,...);
void errormsg(const char *format,...);
char *xstrdup(const char *s);

/* debug */
void dprintf(const char *format,...);
void print();

/*
 function prototypes for all global functions
*/

/* math routines */
flt frand();
flt rand1();
flt Gauss_rand();
flt safe_arctangent(flt a, flt b);

/* the real tracing stuff */
flt find_roughness(flt beta, int type);
color shade(intersection * i, Grid_tree * g, flt imp);
color bgcolor(ray * r);
color fogsources(ray * r, flt endpoint);
color trace(ray & r, Grid_tree * g, flt imp);
intersection all_intersections(ray & r);
bool shadow_test(ibuf * queue, ray * r);

void postcompute();
void precompute();

void read_TGA_head(FILE * f, int &x, int &y);
pixel_data *get_TGA_pixels(char *fn);
pixel_data *get_GIF_pixels(char *fn);

/* get_new_xxx_object */
object *get_new_cone_object(vector,vector,flt,flt);
object *get_new_disc_object(vector,vector,flt,flt= -1);
object *get_new_smooth_triangle_object(vector,vector,vector,vector,vector,vector);
object *get_new_triangle_object(vector,vector,vector);
object *get_new_extrusion_object();
object *get_new_torus_object(flt, flt);
object *get_new_superq_object(vector v);
object *get_new_sphere_object(vector, flt);
object *get_new_quadric_object(vector a,vector b,vector c,flt r);
object *get_new_plane_object(vector, flt);
object *get_new_box_object(vector a,vector b);
object *get_new_inter_object();
object *get_new_union_object();
object *get_new_algebraic_object(svec<poly_instruction>);
object *get_new_polygon_object(svec<vector>);
object *get_new_conic_object();
object *get_new_composite_object();

/*  */
object_list_optimisation *init_BSP(const Optilist &l);

/* lights */
void sample_ray(ray *, flt *, flt, Grid_tree *g);
void init_lightbufs() ;
/*
 * intersection routines; global so they can be used for special bounding
 * shapes
 */

void bound_torus(bounding_box *bb, flt majr, flt minr, const matrix &ctm);
void do_box_bound(bounding_box *, matrix *trans, vector p1, vector p2);
int intersect_rawbox(vector p1, vector p2, ray * r, flt * hits);
int intersect_sphere(vector center, flt rsq, ray * r, flt * hits);


#include "macro.hh"

#endif
