 %{
/* rayparse.y -- a Bison definition file for the Rayce language.

   (c) 1993, 1996 Han-Wen Nienhuys <hanwen@stack.urc.tue.nl>
*/

/*
* August 1993
*
* How does it work??
*
* I dunno, Bison does the difficult stuff.
*
* most of the elements of a PoV/Rayce file consist of an keyword xxx and
* some info enclosed in {}.  In my grammar, this is called a xxx_block,
* which has the value of a pointer to an xxx.  The pointer has space
* alloced, so it suffices to copy the pointer, not it's contents.
*
* Stuff enclosed in the { } of a xxx_block is called the xxx_body.
*/

#include "object.hh"
#include "matrix.hh"
#include "shapes.hh"
#include "camera.hh"
#include "texture.hh"
#include "proto.hh"
#include "parse.hh"
#include "extern.hh"
#include "light.hh"
#include "grid.hh"
#include "polyins.hh"
#include "nonprim.hh"

#ifdef DEBUG
#define YYDEBUG 1
#endif


#ifdef __TURBOC__
#define FIXALLOCA
#endif

#ifdef FIXALLOCA

static kill_alloca();
#define alloca allocaa		/* some compilers don't have alloca(), allocaa() is bugfix */
void *allocaa(size_t size);
#endif

#define YYINITDEPTH 50
#include "vray.hh"

svec<poly_instruction> *poly_code;

%}

%union {
  nc_color 	colorval;
  flt 	fltval;
  nc_vector 	vectorval;
  simple_texture	*simpletextureval;
  texture	*textureval;
  char		stringval[200];
  int intval;

  Identifier 	*dectabptr;
  Camera		*cameraval;
  object		*objectval;
  object_list		*objectlval;
  image_map	*imageval;
  pigment_modifier *pigmentval;
  local_finish  *finishval;
  svec<vector> *veclistval;
}

/* not really needed, but this is somewhat more elegant. */
/*%pure_parser*/

/*tokens */

%token	INCLUDE
%token	OPTIONS
%token	DEFAULTTEXT

%token	BACKGROUND ANTIALIAS JITTER
%token	TOLERANCE  DEPTH DEFINE OPTIMISATION
%token	TIME ITERATIONS CUTOFF     BUNCHING LIGHTEPS

%token	OBJECT
%token	COMPOSITE_T

%token	COLOR RGB
%token	RED GREEN BLUE FILTER
%token	ATMOSPHERE FOG

%token	TRANSLATE ROTATE SCALE

%token	TEXTURE
%token	AMBIENT DIFFUSE BRILLIANCE
%token	ROUGHNESS SPECULAR
%token  IOR
%token  REFRACTION REFLECTION
%token  REFRACT_ANGLE REFLECT_ANGLE

%token	MICROFACET ANGLE GEOMETRIC

%token	TARGA GIF PIGMENT FINISH
%token	IMAGE_MAP MAPTYPE ONCE INTERPOLATE USENORMAL
%token	UVRANGE UVOFFSET UVSWAP

%token 	SPHERE_T PLANE_T QUADRIC_T BOX_T LIGHT_SOURCE_T 
%token 	TRIANGLE_T TORUS_T ALGEBRAIC_T POLYGON_T SMOOTH_TRIANGLE_T
%token 	SUPERQ_T EXTRUSION_T CYLINDER_T DISC_T CONIC_T CONE_T
%token 	OPEN

%token  BOUNDED_BY CLIPPED_BY
%token  SPEED
%token	NO_SHADOW NO_REFLECT NO_EYE NO_REFRACT

%token 	CAMERA_T
%token  DIRECTION LOCATION
%token  SKY FOV
%token	LOOK_AT ASPECT

%token	DEFINE
%token	Z_T X_T Y_T

%token	RADIUS ATTENUATION
%token	SPOTLIGHT TIGHTNESS POINT_AT FOGSOURCE
%token	INVERSE INTERSECTION UNION DIFFERENCE
%token	STURM CLOSEDCURVE

%token <fltval> FLOAT_CONST

%token <dectabptr> FLOAT_IDENTIFIER IDENTIFIER
%token <dectabptr> COLOR_IDENTIFIER TEXTURE_IDENTIFIER
%token <dectabptr> VECTOR_IDENTIFIER
%token <dectabptr> CAMERA_IDENTIFIER

%token <dectabptr> OBJECT_IDENTIFIER
%token <dectabptr> IMAGEMAP_IDENTIFIER

%token <stringval> STRING

/* grammar elements */

%type <dectabptr> 	any_identifier old_identifier

%type <colorval> color_spec
%type <fltval> pfloat
%type <vectorval> vexp
%type <fltval> fexp
%type <vectorval> scale_stuff

%type <intval> geotype microfacettype  filter_type

%type <textureval> texture_block
%type <simpletextureval> simple_texture_body
%type <pigmentval> pigment_body
%type <pigmentval> pigment_block
%type <finishval>  finish_block
%type <finishval> finish_body

%type <cameraval> camera_block
%type <cameraval> camera_body

%type <objectval> bound_shape_block clip_shape_block 
%type <objectlval> bounded_by clipped_by bounded_body clipped_body

%type <objectval> sphere_body		sphere_block
%type <objectval> plane_body		plane_block
%type <objectval> quadric_block 	quadric_body
%type <objectval> box_body		box_block
%type <objectval> light_block	 	light_body
%type <objectval> fog_block	 	fog_body
%type <objectval> triangle_body		triangle_block
%type <objectval> disc_body		disc_block
%type <objectval> union_body		union_block
%type <objectval> intersection_block	intersection_body
%type <objectval> difference_block	difference_body
%type <objectval> csg_shape_block
%type <objectval> composite_body	composite_block
%type <objectval> torus_body		torus_block
%type <objectval> algebraic_block	algebraic_body
%type <objectval> superq_block		superq_body
%type <objectval> polygon_block polygon_body
%type <objectval> smooth_triangle_block        smooth_triangle_body
%type <objectval> cone_block		cone_body

%type <objectval> cylinder_block	cylinder_body

/*
%type <objectval> extrusion_block	extrusion_body
%type <objectval> conic_block		conic_body
*/

%type <objectval> object_block 		object_body


%type <objectval> shape_block

%type <imageval> image_map_body		image_map_block
%type <veclistval> vec_list
/* precedences */

%right '='
%left '<' '>'
%left '-' '+'
%left '*' '/'
%left NEG '!'    /* negation--unary minus */
%left VNEG    /* vector negation. needed for reduce/reduce conflicts */
%right '^'    /* exponentiation        */
%%

/****************************************************************
   GRAMMAR
 ****************************************************************/

/* GLOBAL PARTS */

/* frame is the start token */
frame: 	/* empty */		{  }
	| frame shape_block	{	/* read an object at top-level */
		the_scene.add($2);
	}
	| frame camera_block	 	{	/* the scene camera */
		the_scene.set_cam($2);
	}
	| frame declare_block		{ }	/* a declaration */
  	| frame options_block		{ }	/* options {} */
	| frame default_text_block	{ }
 	| frame atmosphere_block 	{ }
	;

default_text_block:
	DEFAULTTEXT '{' simple_texture_body '}'		{
	 the_scene.set_text($3);
        }
	;

filter_type: fexp	{ $$ = int($1); }
	;

options_body:	/* empty */	{ }
	| options_body TIME fexp ',' fexp	     	{
	    the_scene.set_time($3,$5);
	}
	| options_body BACKGROUND vexp color_spec 	{
	    the_scene.set_bg($4,$3);
	}
	| options_body BACKGROUND color_spec 	     	{
	   the_scene.set_bg($3);
	}
        | options_body LIGHTEPS fexp {
	    light_eps = $3;
	}
	| options_body TOLERANCE fexp {
	    tolerance= $3;
	}
	| options_body ITERATIONS fexp	{
	  if (!iter_override)	/* commandline override? */
		the_scene.tries = (int) $3;	/* No, use $3 */
	}
	| options_body DEPTH fexp		{
	  the_scene .max_trace_level = (int) $3; }
	| options_body CUTOFF fexp		{
	  the_scene .depth_cutoff = $3; }
	| options_body ANTIALIAS filter_type	{
	    the_scene .set_filter($3);
	}
        | options_body JITTER fexp	{
	    the_scene.set_jitter($3);

	}
	| options_body BUNCHING fexp		{
	   the_scene.bunching = int($3);
	}
	| options_body OPTIMISATION fexp	{
	     the_scene.set_root_optimisation ((int)$3);
	}
	;

options_block:	OPTIONS '{' options_body '}'	{ /*empty*/ }
	;

camera_body:/* empty */		{
		$$ = new Camera;
	}
	| CAMERA_IDENTIFIER	{
		$$ = new Camera( * ((Camera *) $1->data) );
	}
	| camera_body ASPECT fexp		{ $$->aspect = $3; }
	| camera_body LOOK_AT vexp		{ $$->point($3); }
	| camera_body LOCATION vexp		{ $$->eye = $3;  }
	| camera_body DIRECTION vexp		{ $$->eye_dir = $3; }
	| camera_body SKY  vexp			{ $$->sky = $3; }
	| camera_body FOV fexp			{ $$->set_fov_angle($3); }
	| camera_body TRANSLATE vexp		{ $$->translate($3); }
	| camera_body ROTATE vexp		{ matrix rm; rm.make_rotation($3); $$->rotate(rm); }
	| camera_body scale_stuff		{ $$->scale($2); }
	;

camera_block: CAMERA_T '{' camera_body '}'	{ $$ = $3; }
	;

atmosphere_block: ATMOSPHERE '{' atmosphere_body '}' { }
	;

atmosphere_body: AMBIENT color_spec 		{ the_scene.ambient_light->set($2); }
        | IOR fexp				{ the_scene.atmosphere_ior = $2; }
	;

/****************************************************************
	Basics
 ****************************************************************/

pfloat: FLOAT_CONST		{ $$ = $1; }
	| FLOAT_IDENTIFIER	{ $$ = * (flt *) $1->data; }
	;
vexp: 	'<' fexp ',' fexp ',' fexp '>'	{ $$.set($2, $4, $6); }
	| '<' fexp '>'			{ $$.set($2, $2, $2); }
	| VECTOR_IDENTIFIER		{ $$ = *(vector *) $1->data; }
	| X_T				{ $$.set(1,0,0); }
	| Y_T				{ $$.set(0,1,0); }
	| Z_T				{ $$.set(0,0,1); }
	| VECTOR_IDENTIFIER '=' vexp	{ $$ = $3; *((vector *) $1->data) = $3; }
	| '<' color_spec '>'		{ $$.set($2.r(), $2.g(), $2.b()); }
	| vexp '+' vexp			{ $$  =$1+ $3; }
	| '-' vexp %prec NEG		{ $$ =-$2; }
	| vexp '-' vexp			{ $$ =$1- $3; }
	| fexp '*' vexp			{ $$ =$1* $3; }
	| vexp '*' fexp			{ $$= $3* $1; }
	| vexp '^' vexp			{ $$= $1^$3; }
	| vexp '/' fexp			{
	  if ($3 == 0.0)
	    yyerror("division by zero");
	  $$= $1/$3;
	}
	| '(' vexp ')'			{ $$ = $2; }
	;


fexp:	'-' fexp %prec VNEG	{ $$ = -$2; }
	| FLOAT_IDENTIFIER '=' fexp	{ $$ = *(flt *) $1->data = $3; }
	| pfloat		{ $$ = $1; }
	|  '(' vexp ',' vexp ')'	{ $$ = $2*$4; }
	|  '|' vexp '|'		{ $$ = $2.length(); }
	| fexp '+' fexp		{ $$ = $1 + $3; }
	| fexp '*'fexp		{ $$ = $1 * $3; }
	| fexp '-' fexp		{ $$ = $1 - $3; }
	| '(' fexp  ')'		{ $$ = $2; }
	| fexp '/' fexp		{
	  if ($3 == 0.0)
	    yyerror("division by zero");
	  $$ = $1 / $3;
  	}
	| fexp '^' fexp		{
	   $$ = pow($1, $3);
	}
	;

scale_stuff:
	SCALE vexp		{
	    $$ = $2;
	    if (!$$.x || !$$.y || !$$.z)
		yyerror("can't scale by 0");
	}
	;

/***************************************************************************
 * TEXTURES								   *
 ***************************************************************************/

/* stricly speaking, "color" for specifying black is correct. ugh */
color_spec: COLOR			{ $$.set(0,0,0); }
	| color_spec RED fexp		{ $$.r() =$3; }
	| color_spec GREEN fexp		{ $$.g() = $3; }
	| color_spec BLUE fexp		{ $$.b() = $3; }
        | color_spec COLOR_IDENTIFIER	{ $$ = *(color *) $2->data; }
	| color_spec FILTER fexp	{  }
	| color_spec RGB vexp		{ $$.set($3.x, $3.y, $3.z); }
	;

image_map_body:
	TARGA STRING	{
	  $$ = new image_map(get_TGA_pixels($2));
	}
	| GIF STRING	{
	  $$ = new image_map(get_GIF_pixels($2));
	}
 	| IMAGEMAP_IDENTIFIER 		{
	  $$ = new image_map( * (image_map *) $1->data);
	}
 	| image_map_body MAPTYPE fexp	{
	    $$->set_type(int($3));
	}
        | image_map_body INTERPOLATE fexp      {
	    $$->interpolated = bool ($3);
	}
	| image_map_body USENORMAL		{ $$->usenormal = true; }
	| image_map_body ONCE			{ $$->once = true; }
	| image_map_body UVRANGE fexp ',' fexp	{ $$->u_range = $3; $$->v_range = $5; }
	| image_map_body UVOFFSET fexp ',' fexp	{ $$->u_offs = $3; $$->v_offs = $5; }
	| image_map_body UVSWAP 		{ $$->uvswap = true; }
	;

geotype: GEOMETRIC fexp { $$ = int ($2); }
        ;

microfacettype:
        MICROFACET fexp		{  $$ = int($2); }
	;

simple_texture_body: /* empty */	{
	  $$ = new simple_texture;
	}
	| TEXTURE_IDENTIFIER	{
	  $$ = new simple_texture(* (simple_texture*) $1->data);
	}
	| simple_texture_body pigment_block 	{ $$->pigment = $2; }
	| simple_texture_body finish_block 	{ $$->finish = $2; }

	| simple_texture_body TRANSLATE vexp	{ ($$)->translate($3); }
        | simple_texture_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
        | simple_texture_body scale_stuff		{ ($$)->scale($2); }
	;

texture_block: 	TEXTURE '{' simple_texture_body '}'	{ $$  = $3; }
	;

image_map_block: IMAGE_MAP	'{' image_map_body '}'	{ $$ = $3; }
	;

pigment_body:
        color_spec		{ $$ = new pov_compatibility($1); }
	| image_map_block	{ $$ = $1; }
	;

finish_body: /* empty */	{
	  $$ = new local_finish;
	}
	| finish_body AMBIENT color_spec	{ $$->ambient = $3; }
	| finish_body AMBIENT fexp		{
	    $$->ambient.set($3);
	}
	| finish_body DIFFUSE color_spec	{ $$->diffuse = $3; }
	| finish_body DIFFUSE fexp		{
	    $$->diffuse .set($3);
	}
	| finish_body ROUGHNESS fexp		{ $$->roughness = $3; }
        | finish_body ANGLE fexp		{ $$->set_rough(degtorad($3)); }
	| finish_body SPECULAR color_spec	{ $$->specular  = $3; }
	| finish_body SPECULAR fexp		{
	    $$->specular .set($3);
	}
	| finish_body IOR fexp			{ $$->index = $3; }
	| finish_body BRILLIANCE fexp		{ $$->brilliance = $3; }
	| finish_body REFLECTION color_spec	{ $$->refl_color = $3; }
	| finish_body REFLECTION fexp		{
	    $$->refl_color.set($3);
	}
	| finish_body REFRACTION color_spec	{ $$->refr_color = $3; }
	| finish_body REFRACTION fexp		{
	    $$->refr_color.set($3);
	}
        | finish_body microfacettype		{ $$->set_geo($2); }
        | finish_body geotype		        { $$->set_distr($2); }
	| finish_body REFRACT_ANGLE fexp	{ $$->refr_diffuse = degtorad($3); }
	| finish_body REFLECT_ANGLE fexp	{ $$->refl_diffuse = degtorad($3); }
	;


/***************************************************************************
 * SHAPES								   *
 ***************************************************************************/

sphere_body: vexp ',' fexp 			{
		$$ = get_new_sphere_object($1, abs($3));
	      }
	      | sphere_body TRANSLATE vexp	{ ($$)->translate($3); }
	      | sphere_body ROTATE vexp 	{
		  matrix rm; rm.make_rotation($3);($$)->rotate(rm);
	      }
	      | sphere_body scale_stuff		{ ($$)->scale($2); }
	      | sphere_body INVERSE		{ $$->flags |= O_INVERTED; }
	      | sphere_body texture_block	{ $$->text = $2; }
	      ;


/* flat and infinite plane : */
plane_body:   vexp ',' fexp 			{
		$$ = get_new_plane_object($1, $3);
	      }
	      | plane_body TRANSLATE vexp	{ ($$)->translate($3); }
	      | plane_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	      | plane_body scale_stuff		{ ($$)->scale($2); }
	      | plane_body INVERSE		{ $$->flags |= O_INVERTED; }
	      | plane_body texture_block	{ $$->text = $2; }
	      ;

quadric_body:
	vexp ',' vexp ',' vexp ',' fexp {
	    $$ = get_new_quadric_object($1, $3, $5, $7);
	}
	| quadric_body TRANSLATE vexp	{ ($$)->translate($3); }
	| quadric_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| quadric_body scale_stuff	{ ($$)->scale($2); }
	| quadric_body INVERSE		{ $$->flags |= O_INVERTED; }
	| quadric_body texture_block	{ $$->text = $2; }
	| quadric_body CLOSEDCURVE	{ $$->flags |=  O_CLOSED;
					  $$->flags &= ~O_INFINITE; }
	;


box_body: vexp ',' vexp		{
		$$ = get_new_box_object($1,$3);
	}
	| box_body TRANSLATE vexp	{ ($$)->translate($3); }
	| box_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| box_body scale_stuff		{ ($$)->scale($2); }
        | box_body INVERSE		{ $$->flags |= O_INVERTED; }
	| box_body texture_block	{ $$->text = $2; }
	;

light_body: vexp	{
          $$ = get_new_light_object();
	  ((light_data *) $$->data)->org = $1;
	}
        | DIRECTION vexp		{
		light_data *l;

		$$ = get_new_light_object();
		$$->flags |= O_NO_SHADOW;
		l = ((light_data *) $$->data);
		l->dir=$2/$2.length();
		l->org= INFTY * l->dir;
	}
	| light_body color_spec		{
		((light_data *) $$->data)->lcolor = $2;
	}
	| light_body RADIUS fexp	{ ((light_data *) $$->data)->radius = $3; }
	| light_body ATTENUATION fexp	{
	    ((light_data *) $$->data)->attenuation_exp = $3;
	}
	| light_body SPOTLIGHT 		{
	    /* do nothing */
	}
	| light_body DIRECTION vexp	{
	    light_data *l = ((light_data *) $$->data);
	    l->dir=$3/$3.length();
	}
	| light_body POINT_AT vexp	{
	    light_data *l = ((light_data *) $$->data);
	    l->dir= $3-l->org;
	    l->dir/=l->dir.length();
	}
        | light_body TIGHTNESS fexp	{
	    ((light_data *) $$->data)->tightness = $3;
	}
	| light_body TRANSLATE vexp	{ ($$)->translate($3); }
	| light_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| light_body scale_stuff	{ ($$)->scale($2); }
	| light_body NO_SHADOW		{ $$->flags ^= O_NO_SHADOW; }
	| light_body NO_REFLECT		{ $$->flags ^= O_NO_REFLECT; }
	;


fog_body: vexp	{
          $$ = get_new_fog_object();
	  ((fog_data *) $$->data)->org = $1;
	}
        | DIRECTION vexp		{
		fog_data *l;

		$$ = get_new_fog_object();
		$$->flags |= O_NO_SHADOW;
		l = ((fog_data *) $$->data);
		l->dir=$2/$2.length();
		l->org= INFTY * l->dir;
	}
	| fog_body color_spec		{
		((fog_data *) $$->data)->lcolor = $2;
	}
	| fog_body RADIUS fexp	{ ((fog_data *) $$->data)->radius = $3; }
	| fog_body ATTENUATION fexp	{
	    ((fog_data *) $$->data)->attenuation_exp = $3;
	}
	| fog_body SPOTLIGHT 		{
	    /* do nothing */
	}
	| fog_body DIRECTION vexp	{
	    fog_data *l = ((fog_data *) $$->data);
	    l->dir=$3/$3.length();
	}
	| fog_body POINT_AT vexp	{
	    fog_data *l = ((fog_data *) $$->data);
	    l->dir = $3-l->org;
	    l->dir /=l->dir.length();
	}
        | fog_body TIGHTNESS fexp	{
	    ((fog_data *) $$->data)->tightness = $3;
	}
	| fog_body TRANSLATE vexp	{ ($$)->translate($3); }
	| fog_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| fog_body scale_stuff	{ ($$)->scale($2); }
	| fog_body NO_SHADOW		{ $$->flags ^= O_NO_SHADOW; }
	| fog_body NO_REFLECT		{ $$->flags ^= O_NO_REFLECT; }
	;



triangle_body:
	vexp ',' vexp ',' vexp {	  
	  $$ = get_new_triangle_object($1,$3,$5);
	}
	| triangle_body TRANSLATE vexp	{ ($$)->translate($3);}
	| triangle_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| triangle_body scale_stuff	{ ($$)->scale($2); }
	| triangle_body texture_block	{ $$->text = $2; }
	;


smooth_triangle_body:
	vexp ',' vexp ',' vexp ',' vexp ',' vexp ',' vexp {
	  $$ = get_new_smooth_triangle_object($1,$3,$5, $7, $9, $11);
	}
	| smooth_triangle_body TRANSLATE vexp	{ ($$)->translate($3);}
	| smooth_triangle_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| smooth_triangle_body scale_stuff	{ ($$)->scale($2); }
	| smooth_triangle_body texture_block	{ $$->text = $2; }
	;


torus_body:
	fexp ',' fexp 	{
	    $$ = get_new_torus_object($1, $3);
	}
	| torus_body TRANSLATE vexp	{ ($$)->translate($3); }
	| torus_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| torus_body scale_stuff	{ ($$)->scale($2); }
	| torus_body INVERSE		{ $$->flags |= O_INVERTED; }
	| torus_body STURM		{ $$->flags |= O_STURM; }
	| torus_body texture_block	{ $$->text = $2; }
	;

/* a nonempty sequence of vectors */
vec_list:
	vexp {
	  $$ = new svec<vector>;
	  $$->add($1);
	}
	| vec_list ',' vexp		{
	  $$->add($3); 
	}
	;

polygon_body:
	vec_list	{
		$$ = get_new_polygon_object(*$1);
		delete $1;
 	}
	| polygon_body TRANSLATE vexp	{ ($$)->translate($3); }
	| polygon_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| polygon_body scale_stuff	{ ($$)->scale($2); }
	| polygon_body texture_block	{ $$->text = $2; }
	;

disc_body:
	vexp ',' vexp ',' fexp 		{
	    $$ = get_new_disc_object($1 , $3, $5);
	}
	| vexp ',' vexp ',' fexp ',' fexp 		{
	    $$ = get_new_disc_object($1 , $3, $5,$7);	    
	}

	| disc_body TRANSLATE vexp	{ ($$)->translate($3); }
	| disc_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| disc_body scale_stuff	{ ($$)->scale($2); }
	| disc_body texture_block	{ $$->text = $2; }
	;
superq_body:	vexp			{ $$ = get_new_superq_object($1); }
	| superq_body TRANSLATE vexp	{ $$->translate($3); }
	| superq_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| superq_body scale_stuff	{ ($$)->scale($2); }
	| superq_body texture_block	{ $$->text = $2; }
	;
/*
extrusion_block: EXTRUSION_T '{' extrusion_body '}'	{ $$ = $3; }
       	;

extrusion_body:	fexp shape_block	{
	  $$ = get_new_extrusion_object();
	  (((extrusion_data *) $$->data)->shape = $2)->daddy = $$;
	  ((extrusion_data *) $$->data)->height = $1;
	}
	| extrusion_body OPEN fexp	{
	  int i = (int) $3;
	  ((extrusion_data *) $$->data)->locap = (i & 0x01);
	  ((extrusion_data *) $$->data)->hicap = (i & 0x02);
	}
	| extrusion_body TRANSLATE vexp	{ ($$)->translate($3); }
	| extrusion_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| extrusion_body scale_stuff	{ ($$)->scale($2); }
	| extrusion_body texture_block	{ $$->text = $2; }
	| extrusion_body INVERSE	{ $$->flags |= O_INVERTED; }
	;

conic_block: CONIC_T '{' conic_body '}'	{ $$ = $3; }
       	;

conic_body:	vexp ',' fexp shape_block	{
	  $$ = get_new_conic_object();
	  (((conic_data *) $$->data)->shape = $4)->daddy = $$;
	  ((conic_data *) $$->data)->apex = $1;
	  	  ((conic_data *) $$->data)->height = $3;

	}
	| conic_body OPEN fexp	{
	  int i = (int) $3;
	  ((conic_data *) $$->data)->locap = (i & 0x01);
	  ((conic_data *) $$->data)->hicap = (i & 0x02);
	}
	| conic_body TRANSLATE vexp	{ ($$)->translate($3); }
	| conic_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| conic_body scale_stuff	{ ($$)->scale($2); }
	| conic_body texture_block	{ $$->text = $2; }
	| conic_body INVERSE		{ $$->flags |= O_INVERTED; }
	;
*/
cylinder_block:	CYLINDER_T '{' cylinder_body '}'	{ $$ = $3; }
	;

cylinder_body:	vexp ',' vexp ',' fexp		{
	  $$ = get_new_cone_object($1,$3,$5,$5);
	}
	| cylinder_body TRANSLATE vexp	{ ($$)->translate($3); }
	| cylinder_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| cylinder_body scale_stuff	{ ($$)->scale($2); }
	| cylinder_body texture_block	{ $$->text = $2; }
	| cylinder_body OPEN fexp	{
	  int i = (int) $3;
	  if (!(((cone_data *) $$->data)->locap = (i & 0x01)) ||
	      !(((cone_data *) $$->data)->hicap = (i & 0x02)))
	  $$->flags &= O_CLOSED;
	}
	| cylinder_body INVERSE		{ $$->flags |= O_INVERTED; }
	;


cone_body:	vexp ',' vexp ',' fexp ',' fexp		{
	  $$ = get_new_cone_object($1,$3,$5,$7);
	}
	| cone_body TRANSLATE vexp	{ ($$)->translate($3); }
	| cone_body ROTATE    vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| cone_body scale_stuff	{ ($$)->scale($2); }
	| cone_body texture_block	{ $$->text = $2; }
	| cone_body OPEN fexp	{
	  int i = (int) $3;
	  if (!(((cone_data *) $$->data)->locap = (i & 0x01)) ||
	      !(((cone_data *) $$->data)->hicap = (i & 0x02)))
	  $$->flags &= O_CLOSED;
	}
	| cone_body INVERSE		{ $$->flags |= O_INVERTED; }
	;


disc_block:	DISC_T	'{' disc_body '}'	{ $$ = $3;}
	;

superq_block:	SUPERQ_T '{' superq_body '}'		{ $$ = $3; }
	;



cone_block:	CONE_T '{' cone_body '}'	{ $$ = $3; }
	;

pigment_block: PIGMENT '{' pigment_body '}'	{ $$ = $3; }
	;

finish_block: FINISH '{' finish_body '}'	{ $$ = $3 }
	;


sphere_block: SPHERE_T '{' sphere_body '}'	{ $$ = $3; }
		;

plane_block: PLANE_T '{' plane_body '}'		{ $$ = $3; }
	      ;

quadric_block: QUADRIC_T '{' quadric_body '}'	{ $$ = $3; }
	;
box_block: BOX_T '{' box_body '}'		{ $$ = $3; }
	;

light_block: LIGHT_SOURCE_T '{' light_body '}'	{ $$ = $3; }
	;



fog_block: FOGSOURCE	'{'	fog_body '}'	{ $$ = $3; }
	;

triangle_block: TRIANGLE_T	'{' triangle_body '}'	{ $$ =$3; }
	;smooth_triangle_block: SMOOTH_TRIANGLE_T	'{' smooth_triangle_body '}'	{ $$ =$3; }
	;

torus_block: TORUS_T	'{' torus_body '}'	{ $$ =$3; }
	;

polygon_block:	POLYGON_T '{' polygon_body '}'	{ $$=  $3; }
	;

algebraic_block: ALGEBRAIC_T	'{' algebraic_body '}'	{ $$ = $3; }
	;

intersection_block: INTERSECTION '{' intersection_body '}'	{ $$ = $3; }
	;

union_block: UNION '{' union_body '}'		{ $$ = $3; }
	;


difference_block: DIFFERENCE '{' difference_body '}'		{ $$ = $3; }
	;


clipped_by:  CLIPPED_BY	'{' clipped_body '}'	{ $$ = $3; }
	;

bounded_by:  BOUNDED_BY	'{' bounded_body '}'	{ $$ = $3; }
	;

composite_block: COMPOSITE_T '{' composite_body '}'	{ $$ = $3; }
	;

/*****************************************************************
 * algebraic surfaces.
 *****************************************************************/


algebraic_body:
	poly_decl	  	{
	  $$ = get_new_algebraic_object(*poly_code);
	}
	| algebraic_body ROTATE vexp    { matrix rm; rm.make_rotation($3); $$->rotate(rm); }
        | algebraic_body scale_stuff	{ ($$)->scale($2); }
	| algebraic_body TRANSLATE vexp { $$->translate ($3); }
	| algebraic_body INVERSE	{ $$->flags |= O_INVERTED; }
	| algebraic_body STURM		{ $$->flags |= O_STURM; }
	| algebraic_body CLOSEDCURVE	{
	    $$->flags |=  O_CLOSED;
	    $$->flags &= ~O_INFINITE;
	}
	| algebraic_body texture_block	{ $$->text = $2; }
	;

poly_decl: poly_start poly3 poly_end    { }
	;

/* start of poly. Needed to clean old code */
poly_start:	'$' 	{ poly_code->clear(); }
	;

poly_end:	'$'	{ }
	;

/*
  parse a polynomial, translate it into code for a stack oriented
  machine. See Kernighan and Pike, the Unix Programming Environment.
*/

poly3:
  pfloat		{
    poly_code->add(poly_instruction($1));
  }
  | '(' poly3 ')'	{
    /* nothing. */
  }
  | X_T			{
      poly_code->add(poly_instruction('x'));
  }
  | Y_T			{
      poly_code->add(poly_instruction('y'));      
  }
  | Z_T			{
      poly_code->add(poly_instruction('z'));      
  }
  | poly3 '+' poly3	{
      poly_code->add(poly_instruction('+'));      
  }
  | poly3 '-' poly3	{
      poly_code->add(poly_instruction('-'));      	    
  }
  | '-' poly3 %prec NEG {
      poly_code->add(poly_instruction('n'));      
  }
  | poly3 '*' poly3	{
      poly_code->add(poly_instruction('*'));      
  }
  | poly3 '^' poly3	{

      if ((*poly_code).last().op  != 'f')
      	  yyerror("polynomial exponentiation must have integer");
      poly_code->add(poly_instruction('^'));      
  }
  | poly3 '=' poly3	{
      poly_code->add(poly_instruction('-'));      
  }
  ;


/***************************************************************************
 * CSG									   *
 ***************************************************************************/


intersection_body:
	csg_shape_block				{
	    $$ = get_new_inter_object();
	    $$->data->add_child($1);
	}
	| intersection_body csg_shape_block	{ $$->data->add_child( $2); }
	| intersection_body INVERSE		{ $$->flags |= O_INVERTED; }
	| intersection_body scale_stuff		{ ($$)->scale($2); }
	| intersection_body ROTATE vexp		{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| intersection_body TRANSLATE vexp	{ ($$)->translate($3); }
	| intersection_body texture_block	{ $$->text = $2; }
	;

difference_body:
	csg_shape_block				{
	    $$ = get_new_inter_object();
	    $$->data->add_child($1);
	
	}
	| difference_body csg_shape_block	{
	    $2->flags = ($2->flags^ O_INVERTED);
    	   $$->data->add_child($2);

	}
	| difference_body INVERSE		{ $$->flags |= O_INVERTED; }
	| difference_body scale_stuff		{ ($$)->scale($2); }
	| difference_body ROTATE vexp		{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| difference_body TRANSLATE vexp	{ ($$)->translate($3); }
	| difference_body texture_block	{ $$->text = $2; }
	;

union_body:
	csg_shape_block				{
	    $$ = get_new_union_object();
	       $$->data->add_child($1);	
	}
	| union_body csg_shape_block	{
	       $$->data->add_child($2);
	}
	| union_body TRANSLATE vexp	{ ($$)->translate($3); }
	| union_body ROTATE vexp	{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| union_body scale_stuff		{ ($$)->scale($2); }
	| union_body INVERSE		{ $$->flags |= O_INVERTED; }
	| union_body texture_block	{ $$->text = $2; }
	;

/*
 * GENERIC OBJECTS
 */


csg_shape_block:
	shape_block	{
	    $$ = $1;
	    if (! ($$->flags & O_CLOSED))
		yyerror("Expecting closed curve");
	}
	;

bound_shape_block:
	shape_block	{
	    $$ = $1;
	    if (! ($$->flags & O_CLOSED))
		yyerror("Expecting closed curve");
	    if ($$->flags & O_INFINITE )
	        yywarning("bounded_by needs finite curve.");

	}
	;

clip_shape_block:
	shape_block	{
	    $$ = $1;
	    if (!($$->flags & O_CLOSED))
		yyerror("Expecting closed curve");
	}
	;


shape_block:
	sphere_block 		{ $$ = $1; }
	| plane_block		{ $$ = $1; }
	| quadric_block 	{ $$ = $1; }
	| torus_block		{ $$ = $1; }
	| box_block		{ $$ = $1; }
	| intersection_block	{ $$ = $1; }
	| difference_block	{ $$ = $1; }
	| union_block		{ $$ = $1; }
	| algebraic_block	{ $$ = $1; }
	| superq_block		{ $$ = $1; }
/*	| extrusion_block	{ $$ = $1; }*/
	| cylinder_block	{ $$ = $1; }
	| polygon_block 	{ $$ = $1; }
	| light_block		{ $$ = $1; }
	| fog_block		{ $$ = $1; }
/*	| conic_block		{ $$ = $1; }*/
	| cone_block		{ $$ = $1; }
	| triangle_block 	{ $$ = $1; }
	| smooth_triangle_block	{ $$ = $1; }
	| disc_block		{ $$ = $1; }
        | object_block	 	{ $$ = $1; }
        | composite_block	{ $$ = $1; }
	;

bounded_body:
        { $$ = new object_list; }
	| bounded_body bound_shape_block	{
	    $$->append_to($2);
	}
	;

clipped_body:
	clip_shape_block		{  $$ = new object_list; }
	| clipped_body clip_shape_block	{	    
	    $$->append_to($2);
	}
	;

composite_body: /* empty */ 	{
            $$ = get_new_composite_object();
	}
	| composite_body shape_block	{
	    $$->data->add_child( $2);
	}
	| composite_body bounded_by	{
	    $$->bounds = *$2;
	    delete $2;
	    $$->flags &=  ~O_INFINITE;
 	}
	| composite_body clipped_by	{
	  $$->clip = *$2;
	  delete $2;
	  $$->flags &= ~O_CLOSED;
	}
	| composite_body clipped_by_bound	{
	  $$->add_bounds_to_clip();
	  $$->flags &= ~O_CLOSED;
	}
	| composite_body TRANSLATE vexp 	{ ($$)->translate($3); }
	| composite_body ROTATE vexp		{ matrix rm; rm.make_rotation($3);($$)->rotate(rm); }
	| composite_body scale_stuff		{ ($$)->scale($2); }
	| composite_body texture_block		{ $$->text = $2; }
	| composite_body SPEED vexp		{ ($$)->add_speed($3); }
	;

clipped_by_bound: CLIPPED_BY '{' BOUNDED_BY '}' 	{ }
	;

object_block: OBJECT '{' object_body '}' 	{ $$ = $3; }
	;

object_body: OBJECT_IDENTIFIER 			{
	  $$ = new object( * ((object *) $1->data));
	}
	| shape_block				{ $$ = $1; }
	| object_body TRANSLATE vexp		{ ($$)->translate($3); }
	| object_body scale_stuff		{ ($$)->scale($2); }
	| object_body ROTATE vexp		{ matrix rm; rm.make_rotation($3); ($$)->rotate(rm); }
	| object_body bounded_by		{
	  $$->bounds = *$2; delete $2;
	  $$->flags &= ~O_INFINITE;
	}
	| object_body clipped_by	{
	  $$->clip = *$2; delete $2;
	  $$->flags &= ~O_CLOSED;
	}
	| object_body clipped_by_bound	{
	  $$->add_bounds_to_clip();
	  $$->flags &= ~O_CLOSED;
	}
	| object_body SPEED vexp		{ $$->add_speed($3); }
	| object_body texture_block		{ $$->text = $2; }
	| object_body INVERSE			{ $$->flags |= O_INVERTED; }
	| object_body STURM			{ $$->flags |= O_STURM; }
	| object_body NO_SHADOW			{ $$->flags |= O_NO_SHADOW; }
	| object_body NO_REFRACT		{ $$->flags |= O_NO_REFRACT; }
	| object_body NO_REFLECT		{ $$->flags |= O_NO_REFLECT; }
	| object_body NO_EYE	 		{ $$->flags |= O_NO_EYE; }
	;


/***************************************************************************
 * DECLARATIONS								   *
 ***************************************************************************/

/* an assignment will be done to this thing. */
any_identifier:
	IDENTIFIER		{ $$ = $1; }
	| old_identifier	{ warning("redeclaration of %s", $1->name);  $$ = $1; }
        ;

/* an old identifier is reused. */
old_identifier:
	FLOAT_IDENTIFIER	{ $$ = $1; }
	| OBJECT_IDENTIFIER	{ $$ = $1; }
	| VECTOR_IDENTIFIER	{ $$ = $1; }
	| CAMERA_IDENTIFIER	{ $$ = $1; }
	| TEXTURE_IDENTIFIER	{ $$ = $1; }
	| COLOR_IDENTIFIER	{ $$ = $1; }
	| IMAGEMAP_IDENTIFIER	{ $$ = $1; }
	;

declare_block:
	DEFINE any_identifier '=' shape_block {
		$2->del_data();

		$2->data = (void*) $4;
		$2->type = OBJECT_IDENTIFIER;
  	}
	| DEFINE any_identifier '=' texture_block	{
		$2->del_data();

		$2->data= (void*) $4;
		$2->type = TEXTURE_IDENTIFIER;
	}
	| DEFINE any_identifier '=' image_map_block	{
		$2->del_data();

		$2->data = (void*) $4;
		$2->type = IMAGEMAP_IDENTIFIER;
	}
	| DEFINE any_identifier '=' color_spec	{

		$2->del_data();

		color  *p = new color;
		$2->data = (void*) p;
		$2->type = COLOR_IDENTIFIER;
		*p = $4;
	}
	| DEFINE any_identifier '=' camera_block	{
		$2->del_data();

		$2->data= (void*) $4;
		$2->type = CAMERA_IDENTIFIER;
	}
	| DEFINE any_identifier '=' vexp	{

		$2->del_data();
		vector *p = new vector;
		$2->data = p;
		p->set($4.x,$4.y,$4.z);
		$2->type = VECTOR_IDENTIFIER;
	}
        | DEFINE any_identifier '=' fexp	{

	    $2->del_data();
	    flt *p = new flt;
	    $2->data= (void*) p;
	    $2->type = FLOAT_IDENTIFIER;
	    *p = $4;
	}
	;

/****************************************************************
  END OF GRAMMAR
 ****************************************************************/

%%

#ifdef FIXALLOCA

PRIVATE *alloca_ptr[300];
PRIVATE int alloca_idx =  0;

void  *allocaa(size_t size)
{
    *p;

    p = xmalloc(size, "allocaa");
    return alloca_ptr[alloca_idx++] = p;
}

PRIVATE kill_alloca()
{

    while (alloca_idx --)
	free(alloca_ptr[alloca_idx]);

}
#endif


// start parsing.
// PRE: the tokenizer is set up
void
start_parse()
{
  poly_code = new svec<poly_instruction>;
#ifdef DEBUG
  yydebug = debug_options & DEBUGPARS;
#endif
  yyparse();

#ifdef FIXALLOCA
   kill_alloca();
#endif 
   delete poly_code;
}
