#ifndef SCENE_HH
#define SCENE_HH
#include "ray.hh"
class Pixel_output;

/// the problem definition
struct Scene {
    Pixel_output *the_output;
    color   *ambient_light;
    color   *background_color;

    /// gradient in which direction?
    vector  bggradient;

    /// number of objects at toplevel 
    int no_obs;
    	
    ////* maximum trace level */
    int max_trace_level;

    /// adaptive depth cutoff threshold 
    flt depth_cutoff;
    /// how much in one bounding volume 
    int bunching;  	

    /// the scene itself. It is a composite object. Everything in the scene is contained in it. */
    object *the_object;
    /// our current camera; there can only be one. */
    Camera *the_cam;     	
    Grid  *the_grid;

    flt *global_ior;
    flt atmosphere_ior;
    int ior_depth;

    /// time information 
    flt start, stop, delta_t;               /* time limits */



    /// current time
    flt  now;

    /// number of tries per pixel 
    int tries;      
    
    //****************
    flt pick_time(int i);
    void read_from(const char *fname);
    ~Scene();
    Scene();
    void render_to(const char *fname)  ;
    void print () const;
    color trace(ray & r, Grid_tree * g, flt importance);
    void set_root_object() ;	
    void set_root_optimisation(int t) ;
    intersection all_intersections(ray & r);
    void add(object *p);
    void set_time(flt t1, flt);
    void set_bg(color c,vector gr = 0.0);
    void set_filter(int i);
    void set_jitter(flt d);
    void set_cam(Camera *c);
    void set_text(texture *c);
    
    void precompute();
    color bgcolor(ray * r);
    color  shade(intersection * i, Grid_tree * g, flt imp);
};

#endif
