
#include "shapeheads.hh"


/// (nonconvex) polygon
struct polygon_data : shape_data {
    MAKESHAPE(polygon);

    void bound(matrix *,bounding_box *m);
    ~polygon_data();
    polygon_data(svec<vector>);
    polygon_data(polygon_data  const &);
    bool all_intersections(ibuf * q, ray * r, int flags, bool & b) const;
    
    vector  normal(vector) const;
    void rotate(matrix const & rotmat);
    void translate(const vector t);
    void scale(const vector s);

private:
    int     no_vertices;        /*@ManMemo: number of vertices */
    vector *vertices;           /*@ManMemo: vertices themselves */
    vector *clips;              /*@ManMemo: inward facing normals */
    vector  normalvector;       /*@ManMemo: normal of the polygon */

    char    project_to;
    vector  bmin,
    bmax;
    flt     mov;                /*@ManMemo: mov (see plane_data) */
    bool    convex;
    bool point_in_concave_polygon(flt t, ray * r) const;
    bool point_in_convex_polygon(flt t, ray * r) const;
};

/*@ManMemo: a (optionally smooth) triangle */
struct triangle_data : shape_data{
    MAKESHAPE(triangle);
    void bound(matrix *,bounding_box *m);

    vector  vertices[3];        /*@ManMemo: vertices of the triangle */
    triangle_data(vector,vector,vector);
    
    bool all_intersections(ibuf * q, ray * r, int flags, bool & b) const;
    vector  normal(vector) const;
    void rotate(matrix const & rotmat);
    void translate(const vector t);
    void scale(const vector s);

protected:
    /*@ManMemo: linear functionals, which compute the components of a vector in the basis formed by
       vert[0]-vert[2], vert[1] - vert[2] */
    vector  linfunc1,
    linfunc2;  
    char    project_to;
    vector  n;                  /*@ManMemo: normal of plane defined by vertices */
    flt     mov;                /*@ManMemo: translate of plane */
    vector  bmin, bmax;
};
///
struct smooth_triangle : triangle_data
{
    vector *normals;                /*@ManMemo: normals; if allocated, then they exist. */
    shape_data *clone() { return new smooth_triangle (*this); }
    void rotate(matrix const & rotmat);
    void scale(const vector s);    
    smooth_triangle(smooth_triangle const &);
    smooth_triangle(vector,vector,vector,vector,vector,vector);
        vector  normal(vector) const;
    void print() const;
    ~smooth_triangle();
};

